@@ -1,4 +1,4 @@
-/*$Id: Firekeeper.cpp,v 1.26 2007/03/23 11:49:03 jwrobel Exp $*/
+/*$Id: Firekeeper.cpp,v 1.28 2007/08/16 12:06:19 jwrobel Exp $*/
 /* ***** BEGIN LICENSE BLOCK *****
  *  This file is part of Firekeeper.
  *
@@ -41,6 +41,10 @@ static const nsIID kIComponentRegistrarIID = NS_ICOMPONENTREGISTRAR_IID;
 static const nsCID kfkHttpHandlerCID  = FK_HTTPHANDLER_CID;
 static const nsCID kfkHttpsHandlerCID = FK_HTTPSHANDLER_CID;
 
+nsDataHashtable<nsISupportsHashKey, fkHttpChannel *> *requestHashTable = nsnull;
+nsDataHashtable<nsISupportsHashKey, fkCallbacks *> *callbacksHashTable = nsnull;
+nsDataHashtable<nsISupportsHashKey, fkLoadGroup *> *groupHashTable = nsnull;
+
 /*Loaded rules*/
 Rules *rules = nsnull;
 
@@ -65,7 +69,7 @@ Firekeeper::~Firekeeper()
 
 NS_IMPL_ISUPPORTS2(Firekeeper, fkIFirekeeper,
 		   nsIObserver)
-	
+
 /*
   Return global Firekeeper object (singleton)
  */
@@ -73,109 +77,129 @@ Firekeeper*
 Firekeeper::GetInstance()
 {
 	static Firekeeper *firekeeper;
-	
+
 	TRACE("");
 	if (firekeeper == nsnull){
 		firekeeper = new Firekeeper();
 		if (!firekeeper)
 			return nsnull;
-		
+
 		/*don't destroy firekeeper*/
-		NS_ADDREF(firekeeper);	
-			
+		NS_ADDREF(firekeeper);
+
 		nsresult rv = firekeeper->init();
 		if (NS_FAILED(rv)){
 			//TODO: delete firekeeper; ...
 			return nsnull;
 		}
 	}
-	
-	NS_IF_ADDREF(firekeeper);
+
+	NS_ADDREF(firekeeper);
 	return firekeeper;
 }
- 
 
 /*
   Init Firekeeper. Replace oryginal Mozilla's HTTP and HTTPS handlers
   with new ones, able to trace and check traffic.
  */
-NS_IMETHODIMP 
+NS_IMETHODIMP
 Firekeeper::init()
 {
 	TRACE("Firekeeper init() called");
-	rules = nsnull;	
+	rules = nsnull;
 	rules = new Rules();
 	if (!rules || rules->init() < 0)
 		return NS_ERROR_OUT_OF_MEMORY;
-	
-	nsCOMPtr<nsIServiceManager> servman; 
-	nsresult rv = NS_GetServiceManager(getter_AddRefs(servman)); 
+
+	nsresult rv;
+
+	if (!groupHashTable){
+		rv = NS_ERROR_OUT_OF_MEMORY;
+		groupHashTable = new nsDataHashtable<nsISupportsHashKey, fkLoadGroup *>;
+		if (!groupHashTable || NS_FAILED(rv = groupHashTable->Init()))
+			return rv;
+	}
+
+	if (!callbacksHashTable){
+		rv = NS_ERROR_OUT_OF_MEMORY;
+		callbacksHashTable = new nsDataHashtable<nsISupportsHashKey, fkCallbacks *>;
+		if (!callbacksHashTable || NS_FAILED(rv = callbacksHashTable->Init()))
+			return rv;
+	}
+
+	if (!requestHashTable){
+		rv = NS_ERROR_OUT_OF_MEMORY;
+		requestHashTable = new nsDataHashtable<nsISupportsHashKey, fkHttpChannel *>;
+		if (!requestHashTable || NS_FAILED(rv = requestHashTable->Init()))
+			return rv;
+	}
+
+
+	nsCOMPtr<nsIServiceManager> servman;
+	rv = NS_GetServiceManager(getter_AddRefs(servman));
 	if (NS_FAILED(rv)){
 		TRACE("Can't get ServiceManager %08x", rv);
 		return rv;
 	}
 
-	nsCOMPtr<nsIComponentManager> compmgr; 
-	rv = NS_GetComponentManager(getter_AddRefs(compmgr)); 
+	nsCOMPtr<nsIComponentManager> compmgr;
+	rv = NS_GetComponentManager(getter_AddRefs(compmgr));
 	if (NS_FAILED(rv)){
 		TRACE("Can't get ComponentManager");
 		return rv;
 	}
 
-	nsCOMPtr<nsIComponentRegistrar> compreg;	
-	rv = compmgr->QueryInterface(kIComponentRegistrarIID, 
+	nsCOMPtr<nsIComponentRegistrar> compreg;
+	rv = compmgr->QueryInterface(kIComponentRegistrarIID,
 				     getter_AddRefs(compreg));
 	if (NS_FAILED(rv)){
 		TRACE("Can't get ComponentRegistrar");
 		return rv;
 	}
-	
+
 	/*register new HTTP handler*/
 	nsCOMPtr<nsIHttpProtocolHandler> oldHandler;
 	rv = servman->
 		GetServiceByContractID(HTTPHANDLER_CONTRACTID,
 				       NS_GET_IID(nsIHttpProtocolHandler),
 				       getter_AddRefs(oldHandler));
-	
 	if (NS_FAILED(rv))
 		return rv;
-	
+
 	if (!oldHandler){
 		TRACE("oldHandler == NULL");
 		return NS_ERROR_FAILURE;
 	}
 	TRACE("got old http handler %08x", &*oldHandler);
-	
+
 	httpHandler = new fkHttpHandler();
 	if (!httpHandler){
 		TRACE("Can't create httpHandler");
 		return NS_ERROR_OUT_OF_MEMORY;
 	}
-	
+
 	rv = httpHandler->init(oldHandler);
 	if (NS_FAILED(rv)){
 		TRACE("Can't init http handler");
 		return rv;
-	}	
-	
+	}
+
 	NS_ADDREF(httpHandler);
-	
-	fkHttpHandlerFactory *httpHandlerFactory = 
+
+	fkHttpHandlerFactory *httpHandlerFactory =
 		new fkHttpHandlerFactory(httpHandler);
-	
+
 	if (!httpHandlerFactory){
 		TRACE("Can't create httpHandlerFactory");
 		return NS_ERROR_OUT_OF_MEMORY;
 	}
-	NS_ADDREF(httpHandlerFactory);	
-	
-	
-	
+	NS_ADDREF(httpHandlerFactory);
+
 	TRACE("registering %s", HTTPHANDLER_CONTRACTID);
 	rv = compreg->RegisterFactory(kfkHttpHandlerCID,
 				      "fkHttpHandler",
 				      HTTPHANDLER_CONTRACTID,
-				      httpHandlerFactory);	
+				      httpHandlerFactory);
 	if (NS_FAILED(rv)){
 		TRACE("Can't register factory");
 		return rv;
@@ -183,24 +207,23 @@ Firekeeper::init()
 
 	TRACE("Http handler replaced. Getting https handler: %s", HTTPSHANDLER_CONTRACTID);
 
-	
 	/*register new HTTPS handler*/
 	nsCOMPtr<nsIHttpProtocolHandler> oldHttpsHandler;
 	rv = servman->
-		GetServiceByContractID(HTTPSHANDLER_CONTRACTID, 
+		GetServiceByContractID(HTTPSHANDLER_CONTRACTID,
 				       NS_GET_IID(nsIHttpProtocolHandler),
 				       getter_AddRefs(oldHttpsHandler));
-	
+
 	TRACE("got https handler %08x", &*oldHttpsHandler);
 	if (NS_FAILED(rv)){
 		return rv;
 	}
-	
+
 	if (!oldHttpsHandler){
 		TRACE("oldHttpsHandler == NULL");
 		return NS_ERROR_FAILURE;
-	}	
-	
+	}
+
 	TRACE("creating new fkHttpsHandler");
 	httpsHandler = new fkHttpHandler();
 	if (!httpsHandler){
@@ -213,9 +236,8 @@ Firekeeper::init()
 		TRACE("Can't init https handler");
 		return rv;
 	}
-	
+
 	NS_ADDREF(httpsHandler);
-	
 	fkHttpHandlerFactory *httpsHandlerFactory =
 		new fkHttpHandlerFactory(httpsHandler);
 	if (!httpsHandlerFactory){
@@ -223,7 +245,7 @@ Firekeeper::init()
 		return NS_ERROR_OUT_OF_MEMORY;
 	}
 	NS_ADDREF(httpsHandlerFactory);
-	
+
 	rv = compreg->RegisterFactory(kfkHttpsHandlerCID,
 				      "fkHttpsHandler",
 				      HTTPSHANDLER_CONTRACTID,
@@ -234,34 +256,33 @@ Firekeeper::init()
 	}
 
 	TRACE("https handler replaced");
+
 	return NS_OK;
 }
 
 //-----------------------------------------------------------------------------
 // Firekeeper::fkIFirekeeper
 //-----------------------------------------------------------------------------
-
-
-NS_IMETHODIMP 
+NS_IMETHODIMP
 Firekeeper::AddRule(const char *rule, char **errmsg)
-{	
+{
 	char *error = nsnull;
 	TRACE("");
 	if (!errmsg)
 		return NS_ERROR_NULL_POINTER;
-	
+
 	/*TODO: don't create parser here*/
 	RulesParser p;
-	
+
 	if (p.init(&error) < 0){
 		err_msg(error);
 		return NS_ERROR_FAILURE;
 	}
 	Rule *newRule =  p.parseRule(rule, &error);
-	
+
 	if (error){
 		TRACE("error %s", error);
-		*errmsg = nsCRT::strdup(error);	
+		*errmsg = nsCRT::strdup(error);
 		if (!*errmsg)
 			return NS_ERROR_OUT_OF_MEMORY;
 
@@ -270,20 +291,20 @@ Firekeeper::AddRule(const char *rule, char **errmsg)
 		return NS_OK;
 	}
 
-	rules->addRule(newRule);	
+	rules->addRule(newRule);
 	searchAPI->search_compile_all();
-	
+
 	return NS_OK;
 }
 
-NS_IMETHODIMP 
+NS_IMETHODIMP
 Firekeeper::AddRules(const char *rulesbuf, char **errmsg)
 {
 	char *error;
 	TRACE("");
 	if (!errmsg)
 		return NS_ERROR_NULL_POINTER;
-	
+
 	RulesParser p;
 	if (p.init(&error)){
 		err_msg(error);
@@ -291,20 +312,20 @@ Firekeeper::AddRules(const char *rulesbuf, char **errmsg)
 	}
 	p.parseRules(rules, rulesbuf, &error);
 	searchAPI->search_compile_all();
-	
+
 	if (error){
 		TRACE("error %s", error);
-		*errmsg = nsCRT::strdup(error);	
+		*errmsg = nsCRT::strdup(error);
 		if (!*errmsg)
 			return NS_ERROR_OUT_OF_MEMORY;
-		
+
 		/*even when there were parsing errors return NS_OK*/
-	}	
-	
+	}
+
 	return NS_OK;
 }
 
-NS_IMETHODIMP 
+NS_IMETHODIMP
 Firekeeper::SetJudge(fkIJudge *judge)
 {
 	TRACE("");
@@ -312,19 +333,21 @@ Firekeeper::SetJudge(fkIJudge *judge)
 	return NS_OK;
 }
 
-NS_IMETHODIMP 
+NS_IMETHODIMP
 Firekeeper::Enable(PRBool tracingEnabled)
 {
 	TRACE("");
-	if (!httpHandler){
-		TRACE("null httpHandler");
+	if (!httpHandler || !httpsHandler){
+		TRACE("null http(s)Handler");
 		return NS_ERROR_FAILURE;
 	}
 	httpHandler->enableTracing(tracingEnabled);
+	httpsHandler->enableTracing(tracingEnabled);
+
 	return NS_OK;
 }
 
-NS_IMETHODIMP 
+NS_IMETHODIMP
 Firekeeper::IsEnabled(PRBool *tracingEnabled)
 {
 	TRACE("");
@@ -336,13 +359,13 @@ Firekeeper::IsEnabled(PRBool *tracingEnabled)
 	return NS_OK;
 }
 
-NS_IMETHODIMP 
+NS_IMETHODIMP
 Firekeeper::ResetRules()
 {
 	TRACE("");
 	if (rules == nsnull)
 		return NS_ERROR_FAILURE;
-	
+
 	rules->resetRules();
 	TRACE("done");
 	return NS_OK;
@@ -352,13 +375,13 @@ Firekeeper::ResetRules()
 //-----------------------------------------------------------------------------
 // Firekeeper::nsIObserver
 //-----------------------------------------------------------------------------
-NS_IMETHODIMP 
+NS_IMETHODIMP
 Firekeeper::Observe(nsISupports *subject, const char* topic,
-		    const PRUnichar* data) 
+		    const PRUnichar* data)
 {
 	TRACE("");
 	if (!topic || strncmp(topic, "xpcom-startup", 13) != 0){
-		return NS_OK;	
+		return NS_OK;
 	}
 	return NS_OK;
 }
