@@ -50,6 +50,9 @@ fkCallbacks::fkCallbacks(nsILoadGroup *loadGroup, nsIInterfaceRequestor *callbac
 	origWebProgressListener = do_QueryInterface(callbacks, &rv);
 	checkResult(rv, "nsIWebProgressListener");
 
+	origWebProgress = do_QueryInterface(callbacks, &rv);
+	checkResult(rv, "nsIWebProgress");
+	
 	origSecurityEventSink = do_QueryInterface(callbacks, &rv);
 	checkResult(rv, "nsISecurityEventSink");
 
@@ -100,6 +103,8 @@ fkCallbacks::QueryInterface(const nsIID &aIID, void **aResult)
 		foundInterface = NS_STATIC_CAST(fkICallbacksProxy*, this);
 	}else if (aIID.Equals(NS_GET_IID(nsIWebProgressListener)) && origWebProgressListener){
 		foundInterface = NS_STATIC_CAST(nsIWebProgressListener*, this);
+	}else if (aIID.Equals(NS_GET_IID(nsIWebProgress)) && origWebProgress){
+		foundInterface = NS_STATIC_CAST(nsIWebProgress*, this);
 	}else if (aIID.Equals(NS_GET_IID(nsIInterfaceRequestor)) && origCallbacks){
 		foundInterface = NS_STATIC_CAST(nsIInterfaceRequestor*, this);
 	}else if (aIID.Equals(NS_GET_IID(nsISecurityEventSink)) && origSecurityEventSink){
@@ -428,3 +433,50 @@ fkCallbacks::Select(const PRUnichar *dialogTitle, const PRUnichar *text, PRUint3
 	return origPrompt->Select(dialogTitle, text, count, selectList, outSelection, _retval);
 }
 
+
+//-----------------------------------------------------------------------------
+// fkCallbacks::nsIWebProgress
+//-----------------------------------------------------------------------------
+
+/*TODO: this is quick and dirty fix. Listeners will get reference to original requests not
+  to proxies. */
+NS_IMETHODIMP
+fkCallbacks::AddProgressListener(nsIWebProgressListener *aListener,
+				 PRUint32 aNotifyMask)
+{
+	TRACE("fkCallbacks::nsIWebProgress");
+	FK_ASSERT(origWebProgress);
+	if (!origWebProgress)
+	    return NS_ERROR_FAILURE;
+	return origWebProgress->AddProgressListener(aListener, aNotifyMask);
+}
+
+NS_IMETHODIMP
+fkCallbacks::RemoveProgressListener(nsIWebProgressListener *aListener)
+{
+	TRACE("fkCallbacks::nsIWebProgress");
+	FK_ASSERT(origWebProgress);
+	if (!origWebProgress)
+	    return NS_ERROR_FAILURE;
+	return origWebProgress->RemoveProgressListener(aListener);
+}
+
+NS_IMETHODIMP
+fkCallbacks::GetDOMWindow(nsIDOMWindow **aResult)
+{
+	TRACE("fkCallbacks::nsIWebProgress");
+	FK_ASSERT(origWebProgress);
+	if (!origWebProgress)
+		return NS_ERROR_FAILURE;
+	return origWebProgress->GetDOMWindow(aResult);
+}
+
+NS_IMETHODIMP
+fkCallbacks::GetIsLoadingDocument(PRBool *aIsLoadingDocument)
+{
+	TRACE("fkCallbacks::nsIWebProgress");
+	FK_ASSERT(origWebProgress);
+	if (!origWebProgress)
+		return NS_ERROR_FAILURE;
+	return origWebProgress->GetIsLoadingDocument(aIsLoadingDocument);
+}
