@@ -26,6 +26,7 @@
 #include "nsISupportsUtils.h"
 
 #include "nsIWebProgressListener.h"
+#include "nsIWebProgress.h"
 #include "nsIInterfaceRequestor.h"
 #include "nsIProgressEventSink.h"
 #include "nsISecurityEventSink.h"
@@ -41,6 +42,7 @@
 #include "Error.h"
 
 class fkCallbacks:  public nsIWebProgressListener
+                  , public nsIWebProgress
                   , public nsIProgressEventSink
                   , public nsISecurityEventSink
                   , public nsIInterfaceRequestor
@@ -53,6 +55,7 @@ class fkCallbacks:  public nsIWebProgressListener
  public:
 	NS_DECL_ISUPPORTS
 	NS_DECL_NSIWEBPROGRESSLISTENER
+	NS_DECL_NSIWEBPROGRESS
 	NS_DECL_NSIPROGRESSEVENTSINK
 	NS_DECL_NSISECURITYEVENTSINK
 	NS_DECL_NSIINTERFACEREQUESTOR
@@ -75,6 +78,7 @@ private:
         nsCOMPtr<nsILoadGroup> parrentGroup;
 	
 	nsCOMPtr<nsIWebProgressListener> origWebProgressListener;
+	nsCOMPtr<nsIWebProgress> origWebProgress;
         nsCOMPtr<nsISecurityEventSink> origSecurityEventSink;
         nsCOMPtr<nsIProgressEventSink> origProgressEventSink;
 	nsCOMPtr<nsIHttpEventSink> origHttpEventSink;
