@@ -1,4 +1,4 @@
-/*$Id: fkRuleReference.h,v 1.5 2006/11/04 21:09:25 jwrobel Exp $*/
+/*$Id: fkRuleReference.h,v 1.6 2007/09/17 22:18:56 jwrobel Exp $*/
 
 /* ***** BEGIN LICENSE BLOCK *****
  *  This file is part of Firekeeper.
@@ -33,10 +33,13 @@ public:
         NS_DECL_FKIRULEREFERENCE
 
 
-        fkRuleReference(const reference*);
+        fkRuleReference();
 	virtual ~fkRuleReference();
+	NS_IMETHODIMP Init(const reference *ref);
+	
 private:	
-	const reference *ref;
+	char *system;
+	char *refId;
 
 };
 
