@@ -1,4 +1,4 @@
-/*$Id: Error.cpp,v 1.8 2006/10/06 08:01:15 jwrobel Exp $*/
+/*$Id: Error.cpp,v 1.10 2007/08/17 20:40:00 jwrobel Exp $*/
 /* ***** BEGIN LICENSE BLOCK *****
  *  This file is part of Firekeeper.
  *
@@ -25,6 +25,7 @@
 #define MAXLINE 8192
 
 
+const char *__funcname;
 
 /**
  * err_sys - Report fatal error to stderr and and terminate process. 
@@ -127,7 +128,7 @@ void trace_func_windows(const char *fmt, ...)
 	va_start(ap, fmt);
 	vsnprintf(buf, MAXLINE, fmt, ap);
 	va_end(ap);
-	
-	fprintf(stderr, "(some function) "); 
-	fprintf(stderr, "%s\n", buf); 
+
+	fprintf(stderr, "(%s) ", __funcname);
+	fprintf(stderr, "%s\n", buf);
 }
