@@ -1,4 +1,4 @@
-/*$Id: RulesParser.cpp,v 1.23 2007/03/23 11:49:03 jwrobel Exp $*/
+/*$Id: RulesParser.cpp,v 1.24 2007/09/05 21:48:54 jwrobel Exp $*/
 /* ***** BEGIN LICENSE BLOCK *****
  *  This file is part of Firekeeper.
  *
@@ -886,11 +886,13 @@ Rule* RulesParser::parseRule(const char *rulebuf, char **error)
 	FK_ASSERT(error);
 
 	rule = new Rule();
+
 	if (!rule){
 		snprintf(errbuf, ERRBUFLEN, "Not enough memory");
 		*error = errbuf;
 		return NULL;
 	}
+
 	errbuf[0] = 0;
 	
 		
@@ -949,6 +951,13 @@ Rule* RulesParser::parseRule(const char *rulebuf, char **error)
 		
 	TRACE("action: %s", rule->action);
 	TRACE("-----------------------------");
+	
+	rule->ruleraw = strdup(rulebuf);
+	if (!rule->ruleraw){
+		snprintf(errbuf, ERRBUFLEN, "Not enough memory");
+		goto error;
+	}
+	
 	if (strlen(errbuf) == 0){
 		*error = NULL;
 		return rule;
